{% extends 'help_base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load static %}
{% load extras %}

{% block page_head %}
  <link type="text/css" rel="stylesheet"
        href="{% static 'help/help.css' %}">
{% endblock %}

{% block content %}
  <a href="{% url 'help:download-manual' %}" class="btn btn-default pdf"
     role="button">
    {% icon 'download' %}
    {% trans 'Download as PDF' %}
  </a>
  {{ content|safe }}
{% endblock %}
