{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "Error" %}</h2>

  <p>
    {{ message }}
  </p>

{% endblock %}
