{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block content %}

  <h3>{{ title }}</h3>

  <form class="form form-add-client" method="post">
    {% csrf_token %}

    {{ form|bootstrap }}

    {% if next_ip %}
      <div class="form-group">
        <label for="id_next_ip" class="control-label">
          {% trans "IP address that will be assigned to this client" %}
        </label>
        <div>
          <input type="text" id="id_next_ip" class="form-control"
                 value="{{ next_ip }}" disabled="disabled">
        </div>
      </div>
    {% endif %}

    <input type="submit" class="btn btn-primary"
           value="{% trans "Add Client" %}"/>
  </form>

{% endblock %}
