{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}
{% load extras %}

{% block content %}
  <h3>{% trans 'Add custom PageKite service' %}</h3>

  <div class="alert alert-warning d-flex align-items-center" role="alert">
    <div class="me-2">
      {% icon 'exclamation-triangle' %}
      <span class="visually-hidden">{% trans "Caution:" %}</span>
    </div>
    <div>
      {% blocktrans trimmed %}
        Your PageKite frontend server may not support all the protocol/port
        combinations that you are able to define here. For example, HTTPS on
        ports other than 443 is known to cause problems.
      {% endblocktrans %}
    </div>
  </div>

  <form class="form" method="post">
      {% csrf_token %}

      {{ form|bootstrap }}

      <input type="submit" class="btn btn-primary"
             value="{% trans "Submit" %}"/>
  </form>
{% endblock %}
