{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load static %}
{% load extras %}

<p>
  {% if data.state == "waiting" %}
    {% icon 'clock-o' %}
  {% elif data.state == "running" %}
    {% icon 'refresh' class='svg-icon spin' %}
  {% elif data.state == "completed" %}
  {% endif %}

  {{ message }}
</p>

{% if data.state == "completed" %}
  <div class="btn-toolbar">
    {% include 'notifications-dismiss-button.html' with id=id %}
  </div>
{% endif %}
