{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load static %}
{% load extras %}

<div class="card">
  <a href="{{ item.url }}">
    <div class="card-icon">
      {% if 'fa-' in item.icon %}
        {% icon item.icon|slice:"3:" class='card-icon' %}
      {% else %}
        {% if item.app_id %}
          {% icon item.app_id|add:'/icons/'|add:item.icon|add:'.svg' class='card-icon' %}
        {% else %}
          {% icon item.icon class='card-icon' %}
        {% endif %}
      {% endif %}
    </div>
    <div class="card-title">{{ item.name }}</div>
    <div class="card-tags">
      {% for tag in item.tags|slice:":3" %}<span class="tag">{% trans tag %}</span>{% if not forloop.last %}<span class="tag-separator"></span>{% endif %}{% endfor %}
    </div>
  </a>
</div>
