{% extends "base_firstboot.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block body_class %}
no-brand
{% endblock %}

{% block content %}
  <div class="logo">
    <img class="firstboot img-fluid" alt="{{ box_name }}"
         src="{% static 'theme/img/freedombox-logo-standard.svg' %}">
  </div>

  <form class="form form-start text-center" method="post">
    {% csrf_token %}

    {% if show_wizard_password_prompt %}
      {{ form|bootstrap }}
    {% endif %}

    <p>
      <input type="submit" class="btn btn-lg btn-primary"
             value="{% trans "Start Setup" %}"/>
    </p>
  </form>
{% endblock %}
