# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Application manifest for privoxy.
"""

from django.utils.translation import gettext_lazy as _

backup: dict = {}

tags = [_('Ad blocker'), _('Proxy server'), _('Local network')]
