{% extends "base_firstboot.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}
{% load extras %}

{% block content %}
  <h2>{% trans "Administrator Account" %}</h2>

  {% if not admin_users %}
    <p>
      {% blocktrans trimmed %}
        Choose a username and password to access this web interface.
        The password can be changed later. This user will be granted
        administrative privileges. Other users can be added later.
      {% endblocktrans %}
    </p>

    <form class="form form-create" method="post">
      {% csrf_token %}

      {{ form|bootstrap }}

      <input type="submit" class="btn btn-primary"
             value="{% trans "Create Account" %}"/>
    </form>
  {% else %}
    <div class="alert alert-danger d-flex align-items-center" role="alert">
      <div class="me-2">
        {% icon 'exclamation-triangle' %}
        <span class="visually-hidden">{% trans "Caution:" %}</span>
      </div>
      <div>
        {% blocktrans trimmed %}
          An administrator account already exists.
        {% endblocktrans %}
      </div>
    </div>

    <p>
      {% blocktrans trimmed %}
        The following administrator accounts exist in the system.
      {% endblocktrans %}
    </p>

    <ul>
      {% for user in admin_users %}
        <li>{{ user }} </li>
      {% endfor %}
    </ul>

    <p>
      {% blocktrans trimmed %}
        Delete these accounts from command line and refresh the page to create
        an account that is usable with {{ box_name }}. On the command line run
        the command "echo '{"args": ["USERNAME", "AUTH_USER", "AUTH_PASSWORD"],
        "kwargs": {}}' | sudo freedombox-cmd users remove_user". If an account
        is already usable with {{ box_name }}, skip this step.
      {% endblocktrans %}
    </p>

    <form class="form form-skip" method="post">
      {% csrf_token %}

      <input type="submit" class="btn btn-link" name="skip"
             value="{% trans "Skip this step" %}"/>
    </form>
  {% endif %}
{% endblock %}
