# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Application manifest for configure.
"""

from django.utils.translation import gettext_lazy as _

tags = [_('Homepage'), _('Logging'), _('Advanced apps')]
