{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load extras %}

{% if clients %}
  {% with client_platforms=clients|clients_get_platforms %}
    {% if client_platforms.web|length == 1 %}
      <a target='_blank' rel="noopener noreferrer"
         class="btn btn-primary" role="button"
         {% if not enabled %} disabled="disabled"
         {% else %} href="{{ client_platforms.web.0.url }}"
         {% endif %}>
        {% trans "Launch web client" %}
        {% icon 'external-link' %}
      </a>
    {% endif %}

    {% if client_platforms.web|length > 1 or client_platforms.other %}
      <button type="button" data-bs-toggle="collapse" data-bs-target="#clients"
              class="btn btn-default collapsed collapsible-button">
        {% icon 'chevron-right' %}
        {% trans "Client Apps" %}
      </button>
    {% endif %}
  {% endwith %}
{% endif %}
