{% extends 'cards.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load static %}
{% load i18n %}

{% block page_js %}
  <script src="{% static 'tags.js' %}" defer></script>
{% endblock %}

{% block body_class %}apps-page{% endblock %}
