{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load extras %}

{% if pkg_manager_is_busy %}
  <div class="alert alert-danger d-flex align-items-center">
    <div class="me-2">
      {% icon 'exclamation-triangle' %}
      <span class="visually-hidden">{% trans "Caution:" %}</span>
    </div>
    <div>
      {% blocktrans trimmed %}
        Currently an installation or upgrade is running. Consider waiting
        until it's finished before shutting down or restarting.
      {% endblocktrans %}
    </div>
  </div>
{% endif %}
