{% extends "base_firstboot.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}
{% load extras %}

{% block body_class %}
no-brand
{% endblock %}

{% block content %}

  <div class="logo">
    <img class="firstboot img-fluid" alt="{{ box_name }}"
         src="{% static 'theme/img/freedombox-logo-standard.svg' %}">
  </div>

  {% if is_first_setup_running %}
    <p class="firstboot-spinner">
      {% icon 'refresh' class='svg-icon spin' %}
    </p>
  {% endif %}

{% endblock %}
