# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Application manifest for users and groups.
"""

from django.utils.translation import gettext_lazy as _

tags = [_('Manage accounts'), _('App permissions')]
