/*
# SPDX-License-Identifier: AGPL-3.0-or-later
*/

.progress {
    margin-bottom: 0;
    width: 11.25rem;
}

.samba-disk-header {
    display: flex;
    justify-content: space-between;
}

.samba-disk-name {
    font-size: 1.3rem;
}

.samba-disk-shares {
    padding-top: 0.625rem;
    margin-bottom: 1.25rem;
}
