{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}
{% load extras %}

{% block content %}

  {% include "app-header.html" with setup=True %}

  {% include "toolbar.html" %}

  {% if setup_state.value == 'up-to-date' %}

     <div class="app-just-installed">
       {% trans "Application installed." %}
     </div>

  {% elif not operations %}

    <p>
      {% if setup_state.value == 'needs-setup' %}
        {% blocktrans trimmed %}
          Install this application?
        {% endblocktrans %}
      {% elif setup_state.value == 'needs-update' %}
        {% blocktrans trimmed %}
          This application needs an update. Update now?
        {% endblocktrans %}
      {% endif %}
    </p>

    <div class="app-unavailable d-none">
      <div class="alert alert-warning d-flex align-items-center" role="alert">
        <div class="me-2">
          {% icon 'exclamation-triangle' %}
          <span class="visually-hidden">{% trans "Caution:" %}</span>
        </div>
        <div>
          {% blocktrans trimmed %}
            This application is currently not available in your distribution.
          {% endblocktrans %}
        </div>
      </div>
    </div>
    <p class="app-checking-availability d-none" data-app-id="{{ app_id }}"
       data-setup-state="{{ setup_state.value }}">
      {% blocktrans trimmed %}
        Checking app availability...
      {% endblocktrans %}
      <span class="spinner-border spinner-border-sm" role="status"></span>
    </p>
    <p class="app-checking-availability-error d-none">
      {% blocktrans trimmed %}
        Error checking app availability. Please refresh page.
      {% endblocktrans %}
    </p>

    <form class="form-install" action="" method="post">
      {% csrf_token %}

      {% if package_conflicts and package_conflicts_action.value != 'ignore' %}
        <div class="alert alert-warning d-flex align-items-center" role="alert">
          <div class="me-2">
            {% icon 'exclamation-triangle' %}
            <span class="visually-hidden">{% trans "Caution:" %}</span>
          </div>
          <div>
            {% blocktrans trimmed %}
              <strong>Conflicting Packages:</strong> Some packages installed on
              the system conflict with the installation of this app. The following
              packages will be removed if you proceed:
            {% endblocktrans %}
            {% for package in package_conflicts %}
              {{ package }}
            {% endfor %}
          </div>
        </div>
      {% endif %}

      <input type="submit" class="btn btn-md btn-primary install-button"
             name="install"
             {% if setup_state.value == 'needs-setup' %}
             value="{% trans "Install" %}"
             {% elif setup_state.value == 'needs-update' %}
             value="{% trans "Update" %}"
             {% endif %} />

    </form>

  {% else %}
    {% include "operations.html" %}
  {% endif %}

{% endblock %}
