{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

<div class="accordion-item">
  <h2 class="accordion-header" id="header-wifi">
    <button class="accordion-button {{ form.errors|yesno:",collapsed" }}"
            type="button" data-bs-toggle="collapse"
            data-bs-target="#collapse-wifi"
            aria-expanded="{{ form.errors|yesno:"true,false" }}"
            aria-controls="collapse-wifi">
      {% trans "Wi-Fi" %}
    </button>
  </h2>

  <div id="collapse-wifi" class="accordion-collapse collapse {{ form.errors|yesno:"show," }}"
       aria-labelledby="header-wifi"
       data-bs-parent="#accordion-connections-fields">
    <div class="accordion-body field-group">
      {{ form.ssid|bootstrap }}
      {{ form.mode|bootstrap }}
      {{ form.band|bootstrap }}
      {{ form.channel|bootstrap }}
      {{ form.bssid|bootstrap }}
      {{ form.auth_mode|bootstrap }}
      {{ form.passphrase|bootstrap }}
    </div>
  </div>
</div>
