{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}
{% load extras %}

<header class="app-header {% if not app_info.icon_filename %} app-header-single-column {% endif %}">

  {% if app_info.icon_filename %}

    {% if app_info.app_id %}
      {% icon app_info.app_id|add:'/icons/'|add:app_info.icon_filename|add:'.svg' %}
    {% else %}
      {% icon app_info.icon_filename %}
    {% endif %}

  {% endif %}

  <section class="app-description" >
    <section class='app-titles'>
      {% block pagetitle %}
        <div>
          {% if setup %}
            <h2>{% trans "Installation" %}: {{ app_info.name }}</h2>
          {% else %}
            <h2>{{ app_info.name }}</h2>
          {% endif %}
        </div>
      {% endblock %}
      {% if app_enable_disable_form %}
        <form class="form form-app-enable-disable" method="post">
          {% csrf_token %}

          {{ app_enable_disable_form|bootstrap }}

          <button name="app_enable_disable_button" type="submit"
            class="btn toggle-button {{ is_enabled|yesno:'toggle-button--toggled,' }}">
          </button>
        </form>
      {% endif %}
    </section>

    {% if app_info.tags %}
      <div class="app-tags d-flex align-items-center flex-wrap">
        {% for tag in app_info.tags %}
        {% if not forloop.first %}•{% endif %}
          <a href="{{ active_section_url }}?tag={{ tag|urlencode }}"
             class="btn btn-default rounded-pill tag">
            {% trans tag %}
          </a>
        {% endfor %}
      </div>
    {% endif %}

    {% block description %}
      {% for paragraph in app_info.description %}
        <p>{{ paragraph|safe }}</p>
      {% endfor %}
    {% endblock %}

    {% if app_info.manual_page %}
      <p class="manual-page">
        <a href="{% url 'help:manual-page' lang='-' page=app_info.manual_page %}">
          {% trans 'Learn more...' %}
        </a>
      </p>
    {% endif %}
  </section>
</header>
