{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "403 Forbidden" %}</h2>

  <p>
    {% with request_path=request.path %}
      {% blocktrans trimmed %}
        You don't have permission to access {{ request_path }} on this server.
      {% endblocktrans %}
    {% endwith %}
  </p>

{% endblock %}
