{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block content %}

  <h2>{% trans "500" %}</h2>

  <p>
    {% url 'logs' app_id='diagnostics' as logs_url %}
    {% blocktrans trimmed %}
      This is an internal error and not something you caused or can fix. Please
      report the error on the <a
      href="https://salsa.debian.org/freedombox-team/freedombox/issues">bug
      tracker</a> so we can fix it. Also, please attach the <a
      href="{{ logs_url }}">logs</a> to the bug report.
    {% endblocktrans %}
  </p>

{% endblock %}
