{% extends "form.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load static %}

{% block page_js %}
  <script src="{% static 'dynamicdns/dynamicdns.js' %}" defer></script>
{% endblock %}
