/*
# SPDX-License-Identifier: AGPL-3.0-or-later
*/

.mount-error,
.mount-success,
.encrypted {
    padding: 0 0.3125rem;
}

.mount-error {
    color: var(--bs-warning);
}

.mount-success {
    color: var(--bs-body-color);
}

.encrypted {
    color: var(--bs-success);
}

.inline-block {
    display: inline-block;
}

.archive-operations {
    width: 13.5rem;
}
