{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

{% block configuration %}

  {% include "power_busy_warning.html" %}

  <div class="btn-toolbar">
    <a class="btn btn-default btn-md" href="{% url 'power:restart' %}">
      {% trans "Restart" %}</a>

    <a class="btn btn-default btn-md" href="{% url 'power:shutdown' %}">
      {% trans "Shut Down" %}</a>
  </div>

{% endblock %}
