# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Application manifest for networks.
"""

from django.utils.translation import gettext_lazy as _

tags = [_('Internet'), _('Wi-Fi'), _('Local network'), _('Topology')]
