# SPDX-License-Identifier: AGPL-3.0-or-later
"""Application manifest for GNOME."""

from django.utils.translation import gettext_lazy as _

backup: dict = {}

tags = [
    _('Desktop'),
    _('Browser'),
    _('Office suite'),
    _('Software store'),
    _('GUI'),
    _('Graphical apps')
]
