{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load extras %}

{% for operation in operations %}
  <div class="app-operation">
    {% icon 'refresh' class='svg-icon spin' %}
    {{ operation.translated_message }}
  </div>
{% endfor %}
