# SPDX-License-Identifier: AGPL-3.0-or-later

from django.utils.translation import gettext_lazy as _

backup = {'secrets': {'directories': ['/etc/coturn']}, 'services': ['coturn']}

tags = [_('Video conference'), _('STUN'), _('TURN')]
