/*
# SPDX-License-Identifier: AGPL-3.0-or-later

# This file based on example code from Javascript XMPP Client which is
# licensed as follows.
#
# The MIT License (MIT)
#
# Copyright (c) 2014 Klaus Herberth <klaus@jsxc.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
*/

h1 {
    /* From bootstrap 3 .page-header */
    text-align: center;
    padding-bottom: 0.5625rem;
    margin: 2.5rem 0 1.25rem;
    border-bottom: 1px solid #eee;
}

/* Elements to hide on page load */
.logout {
   display: none;
}

/* Server connection status message */
#server-flash {
   margin: 0;
   margin-top: 0.625rem;
}

#server-flash:before {
   content: "\e031";
   position: relative;
   top: 0.0625rem;
   padding-right: 0.3125rem;
   display: inline-block;
   font-family: 'Glyphicons Halflings';
   font-style: normal;
   font-weight: 400;
   line-height: 1;
   -webkit-font-smoothing: antialiased;
}

#server-flash.success:before {
   content: "\e089";
   color: green;
}

#server-flash.fail:before {
   content: "\e088";
   color: red;
}
