{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load extras %}

{% block status %}
  <div class="btn-toolbar">
    <a class="btn btn-default" role="button" href="{% url 'security:report' %}"
       title="{% trans 'Show security report' %}">
      {% icon 'line-chart' %}
      {% trans "Show security report" %}
    </a>
  </div>

  {% if is_backports_requested %}
    <h3>{% trans "Frequent Feature Updates" %}</h3>
    <p>
      {% blocktrans trimmed %}
        Frequent feature updates are activated.
      {% endblocktrans %}
    </p>
    <p>
      {% blocktrans trimmed %}
        Frequent feature updates allow the {{box_name}} Service, plus a very
        limited set of software, to receive new features more frequently (from
        the backports repository). This results in receiving some new features
        within weeks, instead of only once every 2 years or so. Note that
        software with frequent feature updates does not have support from the
        Debian Security Team. Instead, they are maintained by contributors to
        Debian and the {{box_name}} community.
      {% endblocktrans %}
    </p>
  {% endif %}
{% endblock %}
