{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load extras %}

<li id="id_help_menu" class="dropdown nav-item">
  <a href="{% url 'help:index' %}" title="{% trans "Help" %}"
     class="nav-link dropdown-toggle" data-bs-toggle="dropdown"
     role="button" aria-expanded="false" aria-haspopup="true"
     id="id_help_menu_link">
    {% icon 'question-circle' %}
    <span class="nav-text d-md-none">
      {% trans "Help" %}
    </span>
  </a>
  <ul class="dropdown-menu" role="menu" aria-labelledby="id_help_menu_link">
    <li>
      <a href="{% url 'help:manual' %}" class="dropdown-item"
         title="{% trans "Manual" context "User guide" %}">
        {% trans "Manual" context "User guide" %}
      </a>
    </li>
    <li role="separator" class="dropdown-divider d-none d-md-block"></li>
    <li>
      <a href="{% url 'help:support' %}" class="dropdown-item"
         title="{% trans "Get Support" %}">
        {% trans "Get Support" %}
      </a>
    </li>
    <li>
      <a href="{% url 'help:feedback' %}" class="dropdown-item"
         title="{% trans "Submit Feedback" %}">
        {% trans "Submit Feedback" %}
      </a>
    </li>
    <li>
      <a href="{% url 'help:contribute' %}" class="dropdown-item"
         title="{% trans "Contribute" %}">
        {% trans "Contribute" %}
      </a>
    </li>
    <li role="separator" class="dropdown-divider d-none d-md-block"></li>
    <li>
      <a href="{% url 'help:about' %}" class="dropdown-item"
         title="{% trans "About" %}">
        {% trans "About" %}
      </a>
    </li>
  </ul>
</li>
