{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load extras %}

{% block configuration %}

  <h3>{% trans "Users" %}</h3>

  <div class="btn-toolbar">
    <a href="{% url 'users:create' %}" class="btn btn-primary"
       role="button" title="{% trans 'Create User' %}">
      {% icon 'plus' %}
      {% trans 'Create User' %}
    </a>
  </div>

  <div class="row">
    <div class="col-md-4">
      <div class="list-group">
        {% for user in object_list %}
          <div class="list-group-item">
            <a class='user-edit-label primary'
               href="{% url 'users:edit' user.username %}"
               title="{% blocktrans with username=user.username %}Edit user {{ username }}{% endblocktrans %}">
              {{ user.username }}
            </a>

            {% if not user.is_active %}
              {% icon 'ban' %}
            {% endif %}

          </div>
        {% endfor %}
      </div>

    </div>
  </div>

{% endblock %}
