{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load extras %}

{% block breadcrumbs %}
{% endblock %}

{% block content %}

  <h2>{% trans "Setup Complete! Next Steps:" %}</h2>

  <ol class='next-steps'>
    <li>
      {% icon 'refresh' class='svg-icon app-icon' %}
      <div class="step-text">
        {% url 'upgrades:index' as upgrades_url %}
        {% blocktrans trimmed %}
          Automatic <a href="{{ upgrades_url }}" target="_blank">software
          update</a> runs daily by default. For the first time, manually run
          it now.
        {% endblocktrans %}
        <form class="form" method="post" target="_blank"
              action="{% url 'upgrades:upgrade' %}">
          {% csrf_token %}
          <input type="submit" class="btn btn-default"
                 value="{% trans "Update now" %}"/>
        </form>
      </div>
    </li>

    <li>
      {% icon 'eye-slash' class='svg-icon app-icon' %}
      <div class="step-text">
        {% url 'privacy:index' as privacy_url %}
        {% blocktrans trimmed %}
          Review <a href="{{ privacy_url }}" target="_blank">privacy options</a>.
        {% endblocktrans %}
      </div>
    </li>

    <li>
      {% icon 'signal' class='svg-icon app-icon' %}
      <div class="step-text">
        {% url 'networks:index' as networks_url %}
        {% blocktrans trimmed %}
          Review and setup <a href="{{ networks_url }}" target="_blank">network
          connections</a>. Change the default Wi-Fi password, if applicable.
        {% endblocktrans %}
      </div>
    </li>

    <li>
      {% icon 'tags' class='svg-icon app-icon' %}
      <div class="step-text">
        {% url 'names:index' as names_url %}
        {% blocktrans trimmed %}
          Configure a <a href="{{ names_url }}" target="_blank">domain name</a>.
        {% endblocktrans %}
      </div>
    </li>

    <li>
      {% icon 'files-o' class='svg-icon app-icon' %}
      <div class="step-text">
        {% url 'backups:index' as backups_url %}
        {% blocktrans trimmed %}
          Configure and schedule remote
          <a href="{{ backups_url }}" target="_blank">backups</a>.
        {% endblocktrans %}
      </div>
    </li>

    <li>
      {% icon 'th' class='svg-icon app-icon' %}
      <div class="step-text">
        {% url 'apps' as apps_url %}
        {% blocktrans trimmed %}
          Put {{ box_name }} to use by installing
          <a href="{{ apps_url }}" target="_blank">apps</a>.
        {% endblocktrans %}
      </div>
    </li>
  </ul>

{% endblock %}
