{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}
{% load extras %}

{% block configuration %}
  {{ block.super }}

  <h3>{% trans "Manage Libraries" %}</h3>

  <div class="btn-toolbar">
    <a href="{% url 'calibre:create-library' %}" class="btn btn-default"
       role="button" title="{% trans 'Create Library' %}">
      {% icon 'plus' %}
      {% trans 'Create Library' %}
    </a>
  </div>

  <div class="row">
    <div class="col-md-6">
      {% if not libraries %}
        <p>{% trans 'No libraries available.' %}</p>
      {% else %}
        <div id="calibre-libraries" class="list-group list-group-two-column">
          {% for library in libraries %}
            <div class="list-group-item">
              <a class="primary"
                 href="/calibre/#library_id={{ library }}&panel=book_list"
                 title="{% blocktrans %}Go to library {{ library }}{% endblocktrans %}">
                {{ library }}
              </a>

              <a href="{% url 'calibre:delete-library' library %}"
                 class="btn btn-default btn-sm secondary" role="button"
                 title="{% blocktrans %}Delete library {{ library }}{% endblocktrans %}">
                {% icon 'trash-o' %}
              </a>
            </div>
          {% endfor %}
        </div>
      {% endif %}
    </div>
  </div>

{% endblock %}
