{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load extras %}

{% block pagetitle %}
  <h2>{{ name }}</h2>
{% endblock %}


{% block configuration %}
  <h3>{% trans "Configuration" %}</h3>

  <p>
    {% blocktrans trimmed %}
      Matrix service needs to be configured for a domain. Users on other Matrix
      servers will be able to reach users on this server using this domain name.
      Matrix user IDs will look like <em>@username:domainname</em>.
    {% endblocktrans %}
  </p>

  <div class="alert alert-warning d-flex align-items-center" role="alert">
    <div class="me-2">
      {% icon 'exclamation-triangle' %}
      <span class="visually-hidden">{% trans "Caution:" %}</span>
    </div>
    <div>
      {% blocktrans trimmed %}
        <strong>Warning!</strong> Changing the domain name after this step will
        require uninstalling and reinstalling the app which will wipe app's
        data.
      {% endblocktrans %}
    </div>
  </div>

  {% if not domain_names %}
    {% url 'names:index' as config_url %}
    <p>
      {% blocktrans trimmed %}
        No domain(s) are available. <a href="{{ config_url }}">Configure</a>
        at least one domain to be able to use Matrix Synapse.
      {% endblocktrans %}
    </p>
  {% else %}
    <form class="form form-configuration" method="post">
      {% csrf_token %}

      {{ form|bootstrap }}

      <input type="submit" class="btn btn-primary"
             value="{% trans "Update setup" %}"/>
    </form>
  {% endif %}
{% endblock %}
