{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

<h1>
  {% blocktrans trimmed %}
    How is Your {{ box_name }} Connected to the Internet?
  {% endblocktrans %}
</h1>

<p>
  {% blocktrans trimmed %}
    Select an option that best describes how your {{ box_name }} is connected in
    your network. This information is used to guide you with further setup. It can
    be changed later.
  {% endblocktrans %}
</p>
