{% extends 'base.html' %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load static %}
{% load i18n %}

{% block page_js %}
  <script src="{% static 'tags.js' %}" defer></script>
{% endblock %}

{% block body_class %}system-page{% endblock %}

{% block container %}
  <div class="container">
    <div class="row">
      <div class="col-lg-8 offset-lg-2">
        {% include 'messages.html' %}
      </div>
    </div>
  </div>

  {% block tags %}
    {% include "tags.html" %}
  {% endblock %}

  <div class="container card-container">
    {% for section_item in menu_items %}
      <div class="system-section-title">{{ section_item.name }}</div>
      <div class="row">
        <div class="card-list card-list-primary">
          {% for item in section_item.items %}
            {% if advanced_mode or not item.advanced %}
              {% include "card.html" %}
            {% endif %}
          {% endfor %}
        </div>
      </div>
    {% endfor %}
  </div>

{% endblock %}
