{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}

<h3>
  {% blocktrans trimmed %}
    {{ box_name }} Internet Connectivity
  {% endblocktrans %}
</h3>

<p>
  {% blocktrans trimmed %}
    The following best describes how your {{ box_name }} is connected in your
    network. This information is used only to suggest necessary configuration
    actions.
  {% endblocktrans %}
</p>

<p>
  {% if network_topology == "to_router" %}
    {% blocktrans trimmed %}
      Your {{ box_name }} gets its Internet connection from your router via Wi-Fi
      or Ethernet cable. This is a typical home setup.
    {% endblocktrans %}
  {% elif network_topology == "as_router" %}
    {% blocktrans trimmed %}
      Your {{ box_name }} is directly connected to the Internet and all your
      devices connect to {{ box_name }} for their Internet connectivity.
    {% endblocktrans %}
  {% else %}
    {% blocktrans trimmed %}
      Your Internet connection is directly attached to your {{ box_name }} and there
      are no other devices on the network.
    {% endblocktrans %}
  {% endif %}
  <a href="{% url 'networks:network-topology' %}" class="btn btn-default"
     role="button">
    {% trans 'Update...' %}
  </a>
</p>
