{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{# Template to display a drop down button for notifications in the navbar #}

{% load i18n %}
{% load extras %}

{% if notifications %}
  <li class="nav-item notifications-dropdown">
    <a href="#" title="{% trans "Notifications" %}"
       class="nav-link dropdown-toggle" data-bs-toggle="collapse"
       role="button" aria-expanded="false" aria-haspopup="true"
       data-bs-target=".notifications">
      {% icon 'bell-o' %}
      <span class="badge rounded-pill
        {% if notifications_max_severity == 'exception' or notifications_max_severity == 'error' %}
          text-bg-danger
        {% elif notifications_max_severity == 'warning' %}
          text-bg-warning
        {% else %}
          text-bg-info
        {% endif %}">
        {{ notifications|length }}
      </span>
    </a>
  </li>
{% endif %}
