{% extends "base.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{# Template to show an App with operations, used by views.AppOperationsView #}

{% block content %}

  {% include "app-header.html" %}

  {% include "toolbar.html" %}

  {% include "operations.html" %}

{% endblock %}
