{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

<form class="form form-diagnostics-button" method="post"
      action="{% url 'diagnostics:app' app_id %}">
  {% csrf_token %}
  <input type="submit" class="dropdown-item no-running-status"
         value="{% trans "Run Diagnostics" %}"
         {% if not enabled %} disabled="disabled" {% endif %}/>
</form>
