{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load static %}
{% load bootstrap %}
{% load i18n %}

{% block page_head %}
  <link type="text/css" rel="stylesheet"
        href="{% static 'networks/networks.css' %}">
{% endblock %}

{% block configuration %}
  {% include "connections_list.html" %}

  {% include "network_topology_main.html" %}

  {% include "internet_connectivity_main.html" %}

  <h3>{%trans "Advanced" %} </h3>
  <p>
    {% blocktrans trimmed %}
      Advanced networking operations such as bonding, bridging and VLAN
      management are provided by the <a href="/_cockpit/network">Cockpit</a>
      app.
    {% endblocktrans %}
  </p>
{% endblock %}
