/*
# SPDX-License-Identifier: AGPL-3.0-or-later

# This file based on example code from Janus Demos which is licensed as
# follows.
#
# 2014-2025  Meetecho
#
# GPL-3 with OpenSSL exception
# If you modify this Program, or any covered work,
# by linking or combining it with OpenSSL
# (or a modified version of that library),
# containing parts covered by the terms of OpenSSL License,
# the licensors of this Program grant you
# additional permission to convey the resulting work.
# Corresponding Source for a non-source form of such a combination
# shall include the source code for the parts of openssl used
# as well as that of the covered work.
*/

/* Default style removed in bootstrap 4 */
.btn-default {
    --bs-btn-color: #333;
    --bs-btn-bg: #fff;
    --bs-btn-border-color: #ccc;
    --bs-btn-hover-color: #333;
    --bs-btn-hover-bg: #e6e6e6;
    --bs-btn-hover-border-color: #adadad;
    --bs-btn-focus-shadow-rgb: 192, 192, 192;
    --bs-btn-active-color: #333;
    --bs-btn-active-bg: #e6e6e6;
    --bs-btn-active-border-color: #adadad;
    --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
    --bs-btn-disabled-color: #333;
    --bs-btn-disabled-bg: #e6e6e6;
    --bs-btn-disabled-border-color: #adadad;
}

.footer {
    display: none;
}

.hide {
        display: none !important;
}

.z-2 {
        z-index: 2000;
}

.rounded {
        border-radius: 5px;
}

.centered {
        display: block;
        margin: auto;
}

.relative {
        position: relative;
}

.top-left {
        position: absolute;
        top: 0px;
        left: 0px;
}

.top-right {
        position: absolute;
        top: 0px;
        right: 0px;
}

.bottom-left {
        position: absolute;
        bottom: 0px;
        left: 0px;
}

.bottom-right {
        position: absolute;
        bottom: 0px;
        right: 0px;
}

.navbar-brand {
        margin-left: 0px !important;
}

.navbar {
        -webkit-box-shadow: 0px 3px 5px rgba(100, 100, 100, 0.49);
        -moz-box-shadow:    0px 3px 5px rgba(100, 100, 100, 0.49);
        box-shadow:         0px 3px 5px rgba(100, 100, 100, 0.49);
}

.navbar-header {
        padding-left: 40px;
}

.btn-group-xs > .btn, .btn-xs {
        padding: 1px 5px;
        font-size: 12px;
        line-height: 1.5;
        border-radius: 3px;
}

.divider {
        width: 100%;
        text-align: center;
}

.divider hr {
        margin-left: auto;
        margin-right: auto;
        width: 45%;
}

div.no-video-container {
        position: relative;
}

.no-video-icon {
        width: 100%;
        height: 240px;
        text-align: center;
        padding-top: 5rem !important;
}

.no-video-text {
        text-align: center;
        position: absolute;
        bottom: 0px;
        right: 0px;
        left: 0px;
        font-size: 24px;
}

.no-video-text-sm {
        text-align: center;
        position: absolute;
        bottom: 0px;
        right: 0px;
        left: 0px;
        font-size: 16px;
}

.meetecho-logo {
        padding: 12px !important;
}

.meetecho-logo > img {
        height: 26px;
}

pre {
        white-space: pre-wrap;
        white-space: -moz-pre-wrap;
        white-space: -pre-wrap;
        white-space: -o-pre-wrap;
        word-wrap: break-word;
        background-color: #f5f5f5;
}

.bg-gray {
        background-color: #f5f5f5;
}

.januscon {
        font-weight: bold;
        animation: pulsating 1s infinite;
}
@keyframes pulsating {
        30% {
                color: #FFD700;
        }
}

.mute-button {
    position: absolute;
    bottom: 0px;
    left: 0px;
    margin: 15px;
}

.unpublish-button {
    position: absolute;
    bottom: 0px;
    right: 0px;
    margin: 15px;
}

.resolution-label {
    position: absolute;
    bottom: 0px;
    left: 0px;
    margin: 15px;
}

.bitrate-label {
    position: absolute;
    bottom: 0px;
    right: 0px;
    margin: 15px;
}

.simulcast-button-group {
    width: 100%;
}
