# SPDX-License-Identifier: AGPL-3.0-or-later

#
# Generated by Django 1.9 on 2015-12-04 07:27
#
"""
Initial Django migration for FreedomBox to create database tables.
"""

from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    """Initial migration schema for FreedomBox models."""

    initial = True

    dependencies: list = []

    operations = [
        migrations.CreateModel(
            name='KVStore',
            fields=[
                ('key', models.TextField(primary_key=True, serialize=False)),
                ('value_json', models.TextField()),
            ],
        ),
    ]
